/*
=====================================================================
	DRCSTR.H (Version 1.00)
		DRC's string function library interface.
	Copyright (c) 2002 by Daniel R. Collins. All rights reserved.
	Permission is granted to freely copy this document,
		and to alter it in any way for personal use.
=====================================================================
*/

#ifndef _DRCSTR_H
#define _DRCSTR_H

// String basics
void   strshift        (char *s, int n);
void   strinsert       (char *s, const char *src);
void   strreplace      (char *s, const char *src, int n);
char   *strskip        (const char *s, const char *src);

// String justification
void   strPadLeft      (char *s, int n, char c = ' ');
void   strPadRight     (char *s, int n, char c = ' ');
void   strJustCenter   (char *s, int n, char c = ' ');
void   strJustLeft     (char *s, int n, char c = ' ');
void   strJustRight    (char *s, int n, char c = ' ');

// String miscellany
void   strLowerCase    (char *s);
void   strUpperCase    (char *s);
void   strIntComma     (char *s, long n);

// String tokenization
void   strTokDelim     (char d);
int    strTokCount     (const char *s);
void   strTokCopy      (const char *s, int idx, char *dest);
int    strTokValInt    (const char *s, int idx);
double strTokValDbl    (const char *s, int idx);

const
char   *strTokPtr      (const char *s, int idx);

#endif // _DRCSTR_H

/*
---------------------------------------------------------------------
	Note: These functions are not "safe" in that they don't check
for use against overall available buffer length(s). When using
them, be sure that your buffer is sufficiently large (or else
wrap them with length-checking calculations, esp. if dealing with
arbitrary user input).
---------------------------------------------------------------------
*/
