/*
=========================================================================
	STRLIST.CPP (Version 1.01)
		Source code for an immutable string list class.
	Copyright (c) 2003-2004 by Daniel R. Collins. All rights reserved.
  You may freely copy this document, if copyright notice is maintained.
=========================================================================
*/

#include <assert.h>
#include <string.h>
#include "strlist.h"


/*
=====================================================================
	Class StringNode -- one string in a list.
=====================================================================
*/

// Constructor

StringNode::StringNode (const char *s)
	{
	string = new char[strlen(s)+1];
	assert(string);
	strcpy(string, s);
	next = NULL;
	}


// Destructor

StringNode::~StringNode (void)
	{
	delete string;
	delete next;
	}


// Return a pointer to the string

const char *StringNode::getString (void)
	{
	return string;
	}


// Return pointer to next linked node

StringNode *StringNode::getNext (void)
	{
	return next;
	}


// Set pointer to the next linked row

void StringNode::setNext (StringNode *n)
	{
	assert(next == NULL);
	next = n;
	}


/*
=====================================================================
	Class StringList -- a list of many strings.
=====================================================================
*/

// Constructor

StringList::StringList (void)
	{
	numStrings = 0;
	firstString = NULL;
	}


// Destructor

StringList::~StringList (void)
	{
  // Trigger recursive deletion of all strings
  delete firstString;
	}


// Add one string to the list

void StringList::appendString (const char *s)
	{
	StringNode *next = new StringNode(s);
	assert(next);

	if (!firstString)
		firstString = next;
	else
		getNode(numStrings-1)->setNext(next);
	numStrings++;
	}


// Return the number of strings in the list

int StringList::getNumStrings (void)
	{
	return numStrings;
	}


// Return an indexed string in the list

const char *StringList::getString (int i)
	{
	StringNode *n = getNode(i);
	return (n ? n->getString() : NULL);
	}


// Return an indexed node in the list

StringNode *StringList::getNode (int i)
	{
	if (i < 0 || numStrings <= i) return NULL;
	for (StringNode *n = firstString; i > 0; i--)
		n = n->getNext();
	return n;
	}

