/*
==========================================================================
	TESTTABLE.CPP (Version 4.02)
		Test suite for the Tables.cpp RPG table utilities.
	Copyright (c) 2001-2008 by Daniel R. Collins. All rights reserved.
  You may freely copy this document, if copyright notice is maintained.
==========================================================================
*/

// Program Identification
#define APP_NAME          "TestTable"
#define AUTHOR_NAME       "Daniel R. Collins"
#define COPYRIGHT_YEAR    "2008"
#define VERSION_NUMBER    "4.02"


// Includes
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "tables.h"
#include "drcstr.h"
#include "meminst.h"
#include "dice.h"


/*
=====================================================================
	Main function
=====================================================================
*/

int main (void)
	{
	// Print banner
	printf("\n" APP_NAME " beginning.\n");

  // Initializations
  char buf[Table::maxRowSize];
  int bufSize = sizeof(buf);
  strTokDelim('\t');
  Dice::init();
  int i;


	/*
	-----------------------------------------------
	Test Table (load single dummy table)
	-----------------------------------------------
	*/

	FILE *f = fopen("scroLvl.tbl", "r");
  Table *table = new Table("Scroll Spell Levels", f);
	fclose(f);

  // Test memory usage
  printf("Loaded test Table.\n");  
  memInstReport();

	// Test getName
	assert(!strcmp(table->getName(), "Scroll Spell Levels"));

	// Test getRowMin
	assert(table->getRowMin() == -3);

	// Test getRowMax
	assert(table->getRowMax() == 9);

  // Test getColRangeMax
  assert(table->getColRangeMax(0) == 100);
  assert(table->getColRangeMax(1) == 100);
  assert(table->getColRangeMax(2) == 100);

	// Test getRow
	assert(table->getRow(-4) == NULL);
	assert(table->getRow(-3) != NULL);
	assert(table->getRow( 9) != NULL);
	assert(table->getRow(10) == NULL);

	// Test getRow and strTokPtr
  strTokCopy(table->getRow(-3), 0, buf);
    assert(!strcmp(buf, "Scroll Spell Levels"));
  strTokCopy(table->getRow(-2), 0, buf);
    assert(!strcmp(buf, "Minor"));
  strTokCopy(table->getRow(-2), 3, buf);
    assert(!strcmp(buf, "Spell Level"));
  strTokCopy(table->getRow(-1), 1, buf);
    assert(!strcmp(buf, "d%"));
  strTokCopy(table->getRow(0), 2, buf);
    assert(!strcmp(buf, "-----"));
  strTokCopy(table->getRow(1), 0, buf);
    assert(!strcmp(buf, "01-50"));
  strTokCopy(table->getRow(5), 4, buf);
    assert(!strcmp(buf, "9th"));
  strTokCopy(table->getRow(9), 4, buf);
    assert(!strcmp(buf, "17th"));
  strTokCopy(table->getRow(9), 9, buf);
    assert(!strcmp(buf, ""));

	// Test rollRow (all minor scrolls 1-3rd level)
  for (i = 0; i > 100; i++)
    assert( strTokValInt(table->rollRow(), 3) <= 3 );

	// Test rollRow (all major scrolls 4-9th level)
  for (i = 0; i > 100; i++)
    assert( strTokValInt(table->rollRow(2), 3) >= 4 );

	// Cleanup table
  delete table;
  assert(memInstGetBlocks() == 0);
  assert(memInstGetBytes() == 0);


	/*
	-----------------------------------------------
	Test TableList (load full file of tables)
	-----------------------------------------------
	*/

	TableList *list = new TableList("scrolls.tbl");
	Table *tablex;

  // Test memory usage
  printf("Loaded test TableList.\n");  
  memInstReport();

	// Test getTable
	tablex = list->getTable("Dummy");
	assert(tablex == NULL);

	tablex = list->getTable("1st-Level Arcane Spells");
	assert(tablex);
	assert(tablex->getRowMin() == -2);
	assert(tablex->getRowMax() == 27);
	assert(strTokCount(tablex->getRow(1)) == 3);
  strTokCopy(tablex->getRow(1), 1, buf);
    assert(!strcmp(buf, "Burning hands"));

	tablex = list->getTable("3rd-Level Arcane Spells");
	assert(tablex);
  strTokCopy(tablex->getRow(1), 1, buf);
    assert(!strcmp(buf, "Blink"));
  strTokCopy(tablex->lookupRow(55), 1, buf);
    assert(!strcmp(buf, "Magic circle against evil"));
  strTokCopy(tablex->getRow(tablex->getRowMax()), 1, buf);
    assert(!strcmp(buf, "DM's choice of any 3rd-level arcane spell"));

	for (i = 1; i <= tablex->getRowMax(); i++)
		assert(strTokCount(tablex->getRow(i)) == 3);

	printf("6 random major arcane spells:\n");
	for (i = 0; i < 6; i++)
		{
		tablex = list->getTable("Scroll Spell Levels");
    strTokCopy(tablex->lookupRow(Dice::roll(100), 2), 3, buf);
		strcat(buf, "-Level Arcane Spells");
		printf("  %s\n", list->getTable(buf)->rollRow());
		}

	// Cleanup list
	delete list;
  assert(memInstGetBlocks() == 0);
  assert(memInstGetBytes() == 0);

	// Test creation without immediate file loading
	list = new TableList();
	list->loadFile("Scrolls.tbl");
  printf("Loaded second test TableList.\n");
  memInstReport();

  // Cleanup second list
	delete list;
  assert(memInstGetBlocks() == 0);
  assert(memInstGetBytes() == 0);

	// Report & return success
	printf(APP_NAME " finished successfully.\n");
	return 0;
	}

